/*  XRoar - a Dragon/Tandy Coco emulator
 *  Copyright (C) 2003  Ciaran Anscomb
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "video.h"
#include "filer.h"
#include "types.h"

#define FG MAPCOLOUR(255,255,255)
#define BG MAPCOLOUR(0,0,128)

extern Uint8 vdg_alpha[768];

#ifdef GP32
static void drawchar(int x, int y, char c) {
	PIXEL *p = VIDEO_CURRENTBASE + y + (x * XSTEP);
	Uint8 *g;
	int i;
	c &= 0x3f;
	g = &vdg_alpha[c*12+2];
	for (i = 0 ; i < 8; i++) {
		if (g[i] & 0x20) *p = FG;
		if (g[i] & 0x10) *(p+1*XSTEP) = FG;
		if (g[i] & 0x08) *(p+2*XSTEP) = FG;
		if (g[i] & 0x04) *(p+3*XSTEP) = FG;
		if (g[i] & 0x02) *(p+4*XSTEP) = FG;
		p--;
	}
}

static void drawstring(int x, int y, char *s) {
	while (*s) {
		drawchar(x,y,*(s++));
		x += 6;
	}
}

static void highlight_line(int offset) {
	PIXEL *p = VIDEO_CURRENTBASE + (223-offset*8);
	int i;
	for (i = 0; i < 14*6; i++) {
		*(p++) ^= 0xff;
		*(p++) ^= 0xff;
		*(p++) ^= 0xff;
		*(p++) ^= 0xff;
		*(p++) ^= 0xff;
		*(p++) ^= 0xff;
		*(p++) ^= 0xff;
		*(p++) ^= 0xff;
		*(p++) ^= 0xff;
		p += 231;
	}
}

static void drawlist(int base) {
	video_fillrect(0, 48, 84, 192, BG);
	int i;
	for (i = 0; i < 20; i++) {
		if (base < filer_dir_count) {
			if (filer_directory[base].attr.attr & 0x10) {
				drawchar(0,231-i*8,'<');
				drawchar(78,231-i*8,'>');
			}
			drawstring(6,231-i*8,filer_directory[base++].name);
		}
	}
}
#endif

#ifndef GP32
static char fnbuf[256];
#endif

char *filepick(void) {
#ifdef GP32
	Uint32 key, oldkey = ~GpKeyGet(), newkey;
	int base = 0, cur = 0, oldbase, oldcur;
	video_backup(VIDEO_CURRENTBASE);
	while(1) {
		filer_readdir(".");
		drawlist(base);
		highlight_line(cur-base);
		do {
			Uint32 ticks = GpTickCountGet();
			key = GpKeyGet();
			newkey = key & oldkey;
			oldkey = ~key;
			oldcur = cur;
			oldbase = base;
			if (key & GPC_VK_UP) cur--;
			if (key & GPC_VK_DOWN) cur++;
			if (key & GPC_VK_FL) cur -= 10;
			if (key & GPC_VK_FR) cur += 10;
			if (cur < 0) cur = 0;
			if (cur >= filer_dir_count) cur = filer_dir_count - 1;
			while (cur < base) base -= 10;
			while (cur > base+19) base += 10;
			if (cur != oldcur) {
				highlight_line(oldcur-oldbase);
				if (base != oldbase)
					drawlist(base);
				highlight_line(cur-base);
			}
			if (newkey & GPC_VK_START)
				return NULL;
			while (GpTickCountGet()-ticks<400);
		} while (!(key & (GPC_VK_SELECT|GPC_VK_FB)));
		if (!(filer_directory[cur].attr.attr & 0x10)) {
			video_restore(VIDEO_CURRENTBASE);
			return filer_directory[cur].name;
		}
		GpRelativePathSet(filer_directory[cur].name);
	}
#else
	char *cr;
	printf("Filename? ");
	fgets(fnbuf, sizeof(fnbuf), stdin);
	cr = strrchr(fnbuf, '\n');
	if (cr)
		*cr = 0;
	return fnbuf;
#endif
}
