/*  XRoar - a Dragon/Tandy Coco emulator
 *  Copyright (C) 2003  Ciaran Anscomb
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <SDL/SDL.h>

#include "xroar.h"
#include "pia.h"
#include "keyboard.h"
#include "snapshot.h"
#include "logging.h"
#include "types.h"

//Uint32 keyboard_matrix[9][9];
/* This gets populated with whatever we have to & the keyboard row
 * with if the corresponding bit on PIA_0B is cleared.
 * eg, if A is pressed, keyboard_column[1] = ~(1<<2); */
Uint32 keyboard_column[8];

static int control = 0;

void keyboard_init(void) {
	/* Register PIA devices */
	PIA_0B.wrfunc[0] = keyboard_column_write;
	/* Writing to the rows makes little sense, but it might happen */
	PIA_0A.wrfunc[0] = keyboard_row_write;
}

#define KEY_PRESSED(s) keyboard_column[keymap[s][0]] &= ~(1<<keymap[s][1])
#define KEY_RELEASED(s) keyboard_column[keymap[s][0]] |= 1<<keymap[s][1]

void keyboard_poll(void) {
	SDL_Event event;
	SDLKey sym;
	while (SDL_PollEvent(&event) == 1) {
		switch(event.type) {
			case SDL_KEYDOWN:
				sym = event.key.keysym.sym;
				//fprintf(stderr, "DOWN: %d\n", sym);
				switch (sym) {
					case SDLK_UP: KEY_PRESSED(11); break;
					case SDLK_DOWN: KEY_PRESSED(10); break;
					case SDLK_LEFT: KEY_PRESSED(8); break;
					case SDLK_RIGHT: KEY_PRESSED(9); break;
					case SDLK_LCTRL: case SDLK_RCTRL:
							 control = 1; break;
					case SDLK_LSHIFT: case SDLK_RSHIFT:
							 KEY_PRESSED(0); break;
					default: if (sym == SDLK_c && control) exit(0);
						if (sym == SDLK_s && control) { snapshot_state = 1; break; }
						if (sym == SDLK_l && control) { snapshot_state = 2; break; }
						if (sym == SDLK_r && control) { snapshot_state = 3; break; }
#ifdef TRACE
						if (sym == SDLK_p && control) { xroar_dumpstate(); break; }
						if (sym == SDLK_t && control) { trace ^= 1; break; }
#endif
						if (sym < 128 && keymap[sym][0] != 8) {
							KEY_PRESSED(sym);
						}
				}
				break;
			case SDL_KEYUP:
				sym = event.key.keysym.sym;
				//fprintf(stderr, "UP: %d\n", sym);
				switch (sym) {
					case SDLK_UP: KEY_RELEASED(11); break;
					case SDLK_DOWN: KEY_RELEASED(10); break;
					case SDLK_LEFT: KEY_RELEASED(8); break;
					case SDLK_RIGHT: KEY_RELEASED(9); break;
					case SDLK_LCTRL: case SDLK_RCTRL:
							 control = 0; break;
					case SDLK_LSHIFT: case SDLK_RSHIFT:
							 KEY_RELEASED(0); break;
					default:
						if (sym < 128 && keymap[sym][0] != 8) {
							KEY_RELEASED(sym);
						}
						break;
				}
				break;
			default:
				break;
		}
	}
	keyboard_column_write(PIA_0B.output_register);
}

void keyboard_reset(void) {
	int i;
	for (i = 0; i < 8; i++)
		keyboard_column[i] = ~0;
}

void keyboard_column_write(Uint32 mask) {
	Uint32 row = ~0;
	int i;
	for (i = 0; i < 8; i++) {
		if ((mask & (1 << i)) == 0) {
			row &= keyboard_column[i];
		}
	}
	//PIA_TIE_LOW(PIA_0A, row);
	//pia_data_set(&PIA_0A, row);
	PIA_0A.tied_low &= 0x80;
	PIA_0A.tied_low |= row & 0x7f;
}

void keyboard_row_write(Uint32 mask) {
	Uint32 col = ~0;
	int i, j;
	//LOG_WARN0("Write to keyboard row detected.  Shenanigans called.\n");
	for (i = 0; i < 7; i++) {
		if ((mask & (1 << i)) == 0) {
			for (j = 0; j < 8; j++) {
				if ((keyboard_column[j] & ~(1<<i)) == 0) {
					col &= ~(1 << j);
				}
			}
		}
	}
	pia_data_set(&PIA_0B, col);
}
